/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.sf;

import hu.midori.kosmos.model.SfRelease;
import hu.midori.kosmos.protocol.SfService;
import hu.midori.kosmos.server.AbstractKosmosService;
import hu.midori.kosmos.server.util.WebCrawlingUtils;
import hu.midori.kosmos.server.util.UrlUtils;

import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Implementation of the <i>SourceForge</i> service.
 * <p>
 * Uses simple page-scaping techniques to retrieve information
 * directly from the <i>SourceForge</i> webpages.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class SfServiceImpl extends AbstractKosmosService implements SfService {
    private final static Log log = LogFactory.getLog(SfServiceImpl.class);

    @SuppressWarnings("unchecked")
    public List<SfRelease> getFileReleases(String url) {
        log.debug("Returning releases...");

        List<SfRelease> releases = new ArrayList<SfRelease>();
        updateTimestamp();

        try {
            Document dom = WebCrawlingUtils.downloadHtmlDom(new URL(url));

            // scrape "Files" link from the "Summary" page and download it
            Node aNode = WebCrawlingUtils.findDomNodeByAttribute(dom.getElementsByTagName("a"), "href", "/project/showfiles.php?group_id=");
            if(aNode == null)
            	throw new IllegalStateException(String.format("Couldn't find 'Files' link for \"%s\"", url));
            String filesPageUrl = UrlUtils.getBaseUrl(url) + aNode.getAttributes().getNamedItem("href").getNodeValue();
            log.debug(String.format("Going to project file releases page at \"%s\"...", filesPageUrl));
            Document domFilesPage = WebCrawlingUtils.downloadHtmlDom(new URL(filesPageUrl));

            // scrape "frelease" table if the project has releases
            Node releasesTableNode = WebCrawlingUtils.findDomNodeByAttribute(domFilesPage.getElementsByTagName("table"), "id", "frelease");
            if(releasesTableNode != null) {
	            NodeList trNodes = releasesTableNode.getLastChild().getChildNodes();
	            for(int j = 0; j < trNodes.getLength(); j++) {
	                Node trNode = trNodes.item(j);
	                Node a0Node = trNode.getFirstChild().getFirstChild();
	                Node a1Node = trNode.getFirstChild().getNextSibling().getFirstChild();
	                Node tdNode = trNode.getFirstChild().getNextSibling().getNextSibling();
	
	                String packageName = a0Node.getFirstChild().getNodeValue();
	                String packageUrl = UrlUtils.getBaseUrl(url) + a0Node.getAttributes().getNamedItem("href").getNodeValue();
	                String version = a1Node.getFirstChild().getNodeValue();;
	                String versionUrl = UrlUtils.getBaseUrl(url) + a1Node.getAttributes().getNamedItem("href").getNodeValue();
	                Date releaseDate = SfConstants.SF_FILERELEASE_DATEFORMAT.parse(tdNode.getFirstChild().getNodeValue());
	
	                SfRelease release = new SfRelease(packageName, packageUrl, version, versionUrl, releaseDate);
	                log.debug(String.format("Adding file release \"%s\"...", release));
	                releases.add(release);
	            }
	
	            if (releases.isEmpty())
	                throw new IllegalStateException(String.format("Couldn't find file releases at \"%s\"", url));
            }
        } catch(Exception ex) {
            log.error("Unable to scrape", ex);
        }

        return releases;
    }
}
