/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.jira;

import hu.midori.kosmos.model.JiraProject;
import hu.midori.kosmos.protocol.JiraService;
import hu.midori.kosmos.server.AbstractKosmosService;
import hu.midori.kosmos.server.AbstractKosmosServiceTest;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.testng.annotations.Configuration;
import org.testng.annotations.Test;

/**
 * Unit test.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class JiraServiceTest extends AbstractKosmosServiceTest {
    private JiraService service = new JiraWebCrawlingServiceImpl();

    @Configuration(beforeTestMethod=true)
    @Override
    public void setup() {
    	super.setup();
    	
    	setupServiceMockEnvironment((AbstractKosmosService)service);
    }    

    /** Tests with invalid URL. */
    @Test()
    public void testInvalidUrl() {
        assert service.getProjects("http://www.nosuchdomain.com/jira").isEmpty();
    }

    /** Tests public Atlassian databases. */
    @Test()
    public void testAtlassianDatabases() {
        testDatabase("http://jira.atlassian.com/browse/CONF");
        testDatabase("http://jira.atlassian.com/browse/CORE");
        testDatabase("http://jira.atlassian.com/browse/JRA");
        testDatabase("http://jira.atlassian.com/browse/TST");
    }

    /** Tests public Codehaus databases. */
    @Test()
    public void testCodehausDatabases() {
        testDatabase("http://jira.codehaus.org/browse/CONTINUUM");
        testDatabase("http://jira.codehaus.org/browse/PICO");
        testDatabase("http://jira.codehaus.org/browse/XFIRE");
        testDatabase("http://jira.codehaus.org/browse/XSTR");
    }

    /** Tests public JBoss databases. */
    @Test()
    public void testJBossDatabases() {
        testDatabase("http://jira.jboss.com/jira/browse/HIBERNATE");
        testDatabase("http://jira.jboss.com/jira/browse/JBAS");
        testDatabase("http://jira.jboss.com/jira/browse/JBPORTAL");
        testDatabase("http://jira.jboss.com/jira/browse/JBSEAM");
    }

    /** Tests a single database. */
    @Test()
    protected void testDatabase(String url) {
        List<JiraProject> projects = service.getProjects(url);
        assert !projects.isEmpty();

        // test projects
        for(JiraProject project : projects) {
            assert !StringUtils.isBlank(project.getName());
            assert !StringUtils.isBlank(project.getUrl());
            assert !StringUtils.isBlank(project.getKey());
            assert !StringUtils.isBlank(project.getProjectUrl());
            assert !StringUtils.isBlank(project.getDescription());
            assert !StringUtils.isBlank(project.getLead());

            assert project.getOpenIssues() >= 0;
            assert project.getCodingInProgressIssues() >= 0;
            assert project.getReopenedIssues() >= 0;
            assert project.getResolvedIssues() >= 0;
            assert project.getClosedIssues() >= 0;

            assert project.getBlockerOpenIssues() >= 0;
            assert project.getCriticalOpenIssues() >= 0;
            assert project.getMajorOpenIssues() >= 0;
            assert project.getMinorOpenIssues() >= 0;
            assert project.getTrivialOpenIssues() >= 0;
            assert project.getOptionalOpenIssues() >= 0;

            testOpenIssuesPerAssignee(project);

            testZoomableImageLocator(project.getIssuesPerStatusChartLocator());
            testZoomableImageLocator(project.getOpenIssuesPerPriorityChartLocator());
            testZoomableImageLocator(project.getOpenIssuesPerAssigneeChartLocator());

            assert (project.getOpenIssues() + project.getCodingInProgressIssues() + project.getReopenedIssues() + project.getResolvedIssues() + project.getClosedIssues()) == project.getTotalIssues();
        }
    }

    /** Tests the open issues per assignee in a project. */
    @Test()
    protected static void testOpenIssuesPerAssignee(JiraProject project) {
        int openIssuesCalculated = 0;
        for(Map.Entry<String, Integer> openIssuePerAssignee : project.getOpenIssuesPerAssignee()) {
            assert !StringUtils.isBlank(openIssuePerAssignee.getKey());
            assert openIssuePerAssignee.getValue() > 0;
            openIssuesCalculated += openIssuePerAssignee.getValue();
        }
        assert openIssuesCalculated == project.getOpenIssues();
    }
}
